#!/bin/bash
command -v mona >/dev/null 2>&1 || { echo "I require mona but it's not installed.  Aborting." >&2; exit 1; }
command -v g++ >/dev/null 2>&1 || { echo "I require g++ but it's not installed.  Aborting." >&2; exit 1; }

g++ read_Mona_Output_Tree.cpp -O3 -Wall -pedantic -lgmp -lgmpxx -o makeoperator.out

echo "Choose a familly of sets by entering the corresponding number)"
echo "You can either choose to enter a formula by yourself and you will then be asked to give a candidate for the growth rate and you will be able to provide some vectors that should be added to the set X."
echo "If you pick instead one of the pre-configured formula then the growth rate and additional vectors will be provided automatically."
grep "##########" mso_formulas |sed "s/#//g;"| cat -n
read selectedNum

if ! [[ $selectedNum =~ ^[0-9]+$ ]] ; then
   echo "error: Not a number" >&2; exit 1
fi
if (( $selectedNum > $(grep "##########" mso_formulas |wc -l))) ; then
   echo "error: This number is not valid" >&2; exit 1
fi

if (($selectedNum==1)) ; then
  echo "Provide a formula with free variable S. Followed by a line starting with #"
  while true
  do
    read line
    if [[ $line =~ ^#.*  ]]
    then break
    fi
    formula="$formula""\n""$line"
  done < /dev/stdin
else
  printf "You have selected "
  grep "##########"  mso_formulas | sed "s/#//g;$selectedNum q;d"
  formula="$(
  awk 'BEGIN{i=0; b=0}{
      if(i=='"$selectedNum"' && b==0){
        if($1=="#"){
          b=1;
        }else{
          print $0;
        }
      }
      if($1=="##########"){ i++; }
    }' mso_formulas)"
fi


initMona="m2l-tree;

restrict(all1 x : x.0 in $ <=> x.1 in $);

pred rightAncestor(var1 leaf, var1 anc) =  ex2 C: anc in C & all1 x: (x in C & x~=leaf) => x.0 in C;

pred edge(var1 l1, var1 l2) = (ex1 x : rightAncestor(l1, x) & rightAncestor(l2,x.1))|(ex1 x : rightAncestor(l2, x) & rightAncestor(l1,x.1));

pred validVertex(var1 vert) = (vert.0 notin $);

pred validSet(var2 Set) = all1 y: y in Set => validVertex(y);

var2 S;
validSet(S); 
"

echo -e "$initMona$formula" > tmp.mona
mona -w -u -n -q tmp.mona | 
egrep "(\(.*->.*|.*Initial state:.*|.*Accepting.*)" |
sed "s/(/ /g; s/)/ /g; s/,/ /g; s/->/ /g; s/Accepting states://g;s/Initial state:/-1/g"|
awk 'BEGIN{i=0}{if($1==-1){i++;};if(i<=2){print $0}}END{print("-1")}'> clean.tmp

if (($selectedNum==1)) ; then
  ./makeoperator.out <(cat clean.tmp)
else
  values="$(
  awk 'BEGIN{i=0; b=0}{
      if(i=='"$selectedNum"' && $1!="##########"){
        if(b==1){
                  print $0;
        }
        if($1=="#"){
          b=1;
        }
      }
      if($1=="##########"){ i++; }
    }' mso_formulas)"
    echo "$values" | ./makeoperator.out <(cat clean.tmp)
fi


